;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Header

	.equ ramarea, 0x02000000
	.equ CursorX,ramarea+32
	.equ CursorY,ramarea+33

	.org  0x08000000     ; GBA ROM Address starts at 0x08000000       

	b	ProgramStart	;000h    4     ROM Entry Point  (32bit ARM branch opcode, eg. "B rom_start") 
	
;004h    156   Nintendo Logo    (compressed bitmap, required!)
	.byte 0xC8,0x60,0x4F,0xE2,0x01,0x70,0x8F,0xE2,0x17,0xFF,0x2F,0xE1,0x12,0x4F,0x11,0x48     ; C
	.byte 0x12,0x4C,0x20,0x60,0x64,0x60,0x7C,0x62,0x30,0x1C,0x39,0x1C,0x10,0x4A,0x00,0xF0     ; D
    .byte 0x14,0xF8,0x30,0x6A,0x80,0x19,0xB1,0x6A,0xF2,0x6A,0x00,0xF0,0x0B,0xF8,0x30,0x6B     ; E
    .byte 0x80,0x19,0xB1,0x6B,0xF2,0x6B,0x00,0xF0,0x08,0xF8,0x70,0x6A,0x77,0x6B,0x07,0x4C     ; F
    .byte 0x60,0x60,0x38,0x47,0x07,0x4B,0xD2,0x18,0x9A,0x43,0x07,0x4B,0x92,0x08,0xD2,0x18     ; 10
    .byte 0x0C,0xDF,0xF7,0x46,0x04,0xF0,0x1F,0xE5,0x00,0xFE,0x7F,0x02,0xF0,0xFF,0x7F,0x02     ; 11
    .byte 0xF0,0x01,0x00,0x00,0xFF,0x01,0x00,0x00,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x00     ; 12
    .byte 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00     ; 13
    .byte 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00     ; 14
	.byte 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1A,0x9E,0x7B,0xEB     ; 15
	
    ;		123456789012
    .ascii "LEARNASM.NET";0A0h    12    Game Title       (uppercase ascii, max 12 characters)	
    .ascii "0000"	;0ACh    4     Game Code        (uppercase ascii, 4 characters)
    .ascii "00"		;0B0h    2     Maker Code       (uppercase ascii, 2 characters)
	.byte 0x96		;0B2h    1     Fixed value      (must be 96h, required!)
	.byte 0			;0B3h    1     Main unit code   (00h for current GBA models)
	.byte 0			;0B4h    1     Device type      (usually 00h) (bit7=DACS/debug related)
	.space 7		;0B5h    7     Reserved Area    (should be zero filled)
	.byte 0			;0BCh    1     Software version (usually 00h)
	.byte 0			;0BDh    1     Complement check (header checksum, required!)
	.word 0			;0BEh    2     Reserved Area    (should be zero filled)
	.long 0			;0C0h    4     RAM Entry Point  (32bit ARM branch opcode, eg. "B ram_start")
	.byte 0			;0C4h    1     Boot mode        (init as 00h - BIOS overwrites this value!)
	.byte 0			;0C5h    1     Slave ID Number  (init as 00h - BIOS overwrites this value!)
	.space 26		;0C6h    26    Not used         (seems to be unused)
	.long 0			;0E0h    4     JOYBUS Entry Pt. (32bit ARM branch opcode, eg. "B joy_start")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
; Program Start


ProgramStart:
	mov sp,#0x03000000			;Init Stack Pointer
	
	mov r4,#0x04000000  		;DISPCNT -LCD Control
	mov r2,#0x403    			;4= Layer 2 on / 3= ScreenMode 3
	str	r2,[r4]         	

	ldr r1,HelloWorldAddress	;Address of Hello World Message
	bl PrintString				;Show Message

	bl NewLine
	
	bl monitor			;Show Regs
	
	bl NewLine
			
	adr	r0,infloop		;Address
	mov r1,#2			;Lines
	bl MemDump
	
InfLoop:
      b InfLoop			;Halt
	
		
HelloWorldAddress:
	.long HelloWorld			;Pointer to Hello message
	
HelloWorld:
	.byte "Hello World",255
	.align 4
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

NewLine:
	STMFD sp!,{r0-r12, lr}
		mov r3,#CursorX
		mov r0,#0
		strB r0,[r3]	;X
		
		mov r3,#CursorY
		ldrB r0,[r3]	;Y
		add r0,r0,#1
		strB r0,[r3]	;Y
	LDMFD sp!,{r0-r12, pc}
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PrintString:					;Print 255 terminated string 
	STMFD sp!,{r0-r12, lr}
PrintStringAgain:
		ldrB r0,[r1],#1
		cmps r0,#255
		beq PrintStringDone		;Repeat until 255
		bl printchar 			;Print Char
		b PrintStringAgain
PrintStringDone:
	LDMFD sp!,{r0-r12, pc}
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
	
PrintChar:
	STMFD sp!,{r0-r12, lr}
	mov r4,#0
		mov r5,#0
		
		mov r3,#CursorX
		ldrB r4,[r3]			;X pos
		mov r3,#CursorY
		ldrB r5,[r3]			;Y pos
		
		mov r3,#0x06000000 		;VRAM base
		
		mov r6,#16				;Xpos, 2 bytes per pixel, 8 bytes per char
		mul r2,r4,r6
		add r3,r3,r2
		
		mov r4,#240*8*2			;Ypos, 240 pixels per line,
		mul r2,r5,r4				;2 bytes per pixel, 8 lines per char
		add r3,r3,r2
		
		adr r4,BitmapFont 		;Font source
		
		sub r0,r0,#32			;First Char is 32 (space)
		add r4,r4,r0,asl #3		;8 bytes per char
		
		mov r1,#8				;8 lines 
DrawLine:
		mov r7,#8 				;8 pixels per line
		ldrb r8,[r4],#1			;Load Letter
		mov r9,#0b100000000		;Mask
				
		mov r2, #0b1111111101000000; Color: ABBBBBGGGGGRRRRR	A=Alpha
DrawPixel:
		tst r8,r9				;Is bit 1?
		strneh r2,[r3]			;Yes? then fill pixel (HalfWord)
		add r3,r3,#2
		mov r9,r9,ror #1		;Bitshift Mask
		subs r7,r7,#1
		bne DrawPixel			;Next Hpixel
		
		add r3,r3,#480-16	   ;Move Down a line (240 pixels *2 bytes) 
		subs r1,r1,#1								;-1 char (16 px)
		bne DrawLine			;Next Vline
LineDone:	
		mov r3,#CursorX
		ldrB r0,[r3]	
		add r0,r0,#1			;Move across screen
		strB r0,[r3]	
	LDMFD sp!,{r0-r12, pc}

	

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.equ MonitorWidth,6
	.include "/srcALL/V1_Monitor.asm"
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
BitmapFont:
        .byte 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00     ;  0
        .byte 0x10,0x18,0x18,0x18,0x18,0x00,0x18,0x00     ;  1
        .byte 0x28,0x6C,0x28,0x00,0x00,0x00,0x00,0x00     ;  2
        .byte 0x00,0x28,0x7C,0x28,0x7C,0x28,0x00,0x00     ;  3
        .byte 0x18,0x3E,0x48,0x3C,0x12,0x7C,0x18,0x00     ;  4
        .byte 0x02,0xC4,0xC8,0x10,0x20,0x46,0x86,0x00     ;  5
        .byte 0x10,0x28,0x28,0x72,0x94,0x8C,0x72,0x00     ;  6
        .byte 0x0C,0x1C,0x30,0x00,0x00,0x00,0x00,0x00     ;  7
        .byte 0x18,0x18,0x30,0x30,0x30,0x18,0x18,0x00     ;  8
        .byte 0x18,0x18,0x0C,0x0C,0x0C,0x18,0x18,0x00     ;  9
        .byte 0x08,0x49,0x2A,0x1C,0x14,0x22,0x41,0x00     ; 10
        .byte 0x00,0x18,0x18,0x7E,0x18,0x18,0x00,0x00     ; 11
        .byte 0x00,0x00,0x00,0x00,0x00,0x18,0x18,0x30     ; 12
        .byte 0x00,0x00,0x00,0x7E,0x7E,0x00,0x00,0x00     ; 13
        .byte 0x00,0x00,0x00,0x00,0x00,0x18,0x18,0x00     ; 14
        .byte 0x02,0x04,0x08,0x10,0x20,0x40,0x80,0x00     ; 15
        .byte 0x7C,0xC6,0xD6,0xD6,0xD6,0xC6,0x7C,0x00     ; 16
        .byte 0x10,0x18,0x18,0x18,0x18,0x18,0x08,0x00     ; 17
        .byte 0x3C,0x7E,0x06,0x3C,0x60,0x7E,0x3C,0x00     ; 18
        .byte 0x3C,0x7E,0x06,0x1C,0x06,0x7E,0x3C,0x00     ; 19
        .byte 0x18,0x3C,0x64,0xCC,0x7C,0x0C,0x08,0x00     ; 20
        .byte 0x3C,0x7E,0x60,0x7C,0x06,0x7E,0x3E,0x00     ; 21
        .byte 0x3C,0x7E,0x60,0x7C,0x66,0x66,0x3C,0x00     ; 22
        .byte 0x3C,0x7E,0x06,0x0C,0x18,0x18,0x10,0x00     ; 23
        .byte 0x3C,0x66,0x66,0x3C,0x66,0x66,0x3C,0x00     ; 24
        .byte 0x3C,0x66,0x66,0x3E,0x06,0x7E,0x3C,0x00     ; 25
        .byte 0x00,0x00,0x18,0x18,0x00,0x18,0x18,0x00     ; 26
        .byte 0x00,0x00,0x18,0x18,0x00,0x18,0x18,0x30     ; 27
        .byte 0x0C,0x1C,0x38,0x60,0x38,0x1C,0x0C,0x00     ; 28
        .byte 0x00,0x00,0x7E,0x00,0x00,0x7E,0x00,0x00     ; 29
        .byte 0x60,0x70,0x38,0x0C,0x38,0x70,0x60,0x00     ; 30
        .byte 0x3C,0x76,0x06,0x1C,0x00,0x18,0x18,0x00     ; 31
        .byte 0x7C,0xCE,0xA6,0xB6,0xC6,0xF0,0x7C,0x00     ; 32
        .byte 0x18,0x3C,0x66,0x66,0x7E,0x66,0x24,0x00     ; 33
        .byte 0x3C,0x66,0x66,0x7C,0x66,0x66,0x3C,0x00     ; 34
        .byte 0x38,0x7C,0xC0,0xC0,0xC0,0x7C,0x38,0x00     ; 35
        .byte 0x3C,0x64,0x66,0x66,0x66,0x64,0x38,0x00     ; 36
        .byte 0x3C,0x7E,0x60,0x78,0x60,0x7E,0x3C,0x00     ; 37
        .byte 0x38,0x7C,0x60,0x78,0x60,0x60,0x20,0x00     ; 38
        .byte 0x3C,0x66,0xC0,0xC0,0xCC,0x66,0x3C,0x00     ; 39
        .byte 0x24,0x66,0x66,0x7E,0x66,0x66,0x24,0x00     ; 40
        .byte 0x10,0x18,0x18,0x18,0x18,0x18,0x08,0x00     ; 41
        .byte 0x08,0x0C,0x0C,0x0C,0x4C,0xFC,0x78,0x00     ; 42
        .byte 0x24,0x66,0x6C,0x78,0x6C,0x66,0x24,0x00     ; 43
        .byte 0x20,0x60,0x60,0x60,0x60,0x7E,0x3E,0x00     ; 44
        .byte 0x44,0xEE,0xFE,0xD6,0xD6,0xD6,0x44,0x00     ; 45
        .byte 0x44,0xE6,0xF6,0xDE,0xCE,0xC6,0x44,0x00     ; 46
        .byte 0x38,0x6C,0xC6,0xC6,0xC6,0x6C,0x38,0x00     ; 47
        .byte 0x38,0x6C,0x64,0x7C,0x60,0x60,0x20,0x00     ; 48
        .byte 0x38,0x6C,0xC6,0xC6,0xCA,0x74,0x3A,0x00     ; 49
        .byte 0x3C,0x66,0x66,0x7C,0x6C,0x66,0x26,0x00     ; 50
        .byte 0x3C,0x7E,0x60,0x3C,0x06,0x7E,0x3C,0x00     ; 51
        .byte 0x3C,0x7E,0x18,0x18,0x18,0x18,0x08,0x00     ; 52
        .byte 0x24,0x66,0x66,0x66,0x66,0x66,0x3C,0x00     ; 53
        .byte 0x24,0x66,0x66,0x66,0x66,0x3C,0x18,0x00     ; 54
        .byte 0x44,0xC6,0xD6,0xD6,0xFE,0xEE,0x44,0x00     ; 55
        .byte 0xC6,0x6C,0x38,0x38,0x6C,0xC6,0x44,0x00     ; 56
        .byte 0x24,0x66,0x66,0x3C,0x18,0x18,0x08,0x00     ; 57
        .byte 0x7C,0xFC,0x0C,0x18,0x30,0x7E,0x7C,0x00     ; 58
        .byte 0x1C,0x30,0x30,0x30,0x30,0x30,0x1C,0x00     ; 59
        .byte 0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x00     ; 60
        .byte 0x38,0x0C,0x0C,0x0C,0x0C,0x0C,0x38,0x00     ; 61
        .byte 0x18,0x18,0x18,0x18,0x7E,0x7E,0x18,0x18     ; 62
        .byte 0x18,0x18,0x18,0x18,0x3C,0x3C,0x18,0x18     ; 63
        .byte 0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18     ; 64
        .byte 0x00,0x00,0x38,0x0C,0x7C,0xCC,0x78,0x00     ; 65
        .byte 0x20,0x60,0x7C,0x66,0x66,0x66,0x3C,0x00     ; 66
        .byte 0x00,0x00,0x3C,0x66,0x60,0x66,0x3C,0x00     ; 67
        .byte 0x08,0x0C,0x7C,0xCC,0xCC,0xCC,0x78,0x00     ; 68
        .byte 0x00,0x00,0x3C,0x66,0x7E,0x60,0x3C,0x00     ; 69
        .byte 0x1C,0x36,0x30,0x38,0x30,0x30,0x10,0x00     ; 70
        .byte 0x00,0x00,0x3C,0x66,0x66,0x3E,0x06,0x3C     ; 71
        .byte 0x20,0x60,0x6C,0x76,0x66,0x66,0x24,0x00     ; 72
        .byte 0x18,0x00,0x18,0x18,0x18,0x18,0x08,0x00     ; 73
        .byte 0x06,0x00,0x04,0x06,0x06,0x26,0x66,0x3C     ; 74
        .byte 0x20,0x60,0x66,0x6C,0x78,0x6C,0x26,0x00     ; 75
        .byte 0x10,0x18,0x18,0x18,0x18,0x18,0x08,0x00     ; 76
        .byte 0x00,0x00,0x6C,0xFE,0xD6,0xD6,0xC6,0x00     ; 77
        .byte 0x00,0x00,0x3C,0x66,0x66,0x66,0x24,0x00     ; 78
        .byte 0x00,0x00,0x3C,0x66,0x66,0x66,0x3C,0x00     ; 79
        .byte 0x00,0x00,0x3C,0x66,0x66,0x7C,0x60,0x20     ; 80
        .byte 0x00,0x00,0x78,0xCC,0xCC,0x7C,0x0C,0x08     ; 81
        .byte 0x00,0x00,0x38,0x7C,0x60,0x60,0x20,0x00     ; 82
        .byte 0x00,0x00,0x3C,0x60,0x3C,0x06,0x7C,0x00     ; 83
        .byte 0x10,0x30,0x3C,0x30,0x30,0x3E,0x1C,0x00     ; 84
        .byte 0x00,0x00,0x24,0x66,0x66,0x66,0x3C,0x00     ; 85
        .byte 0x00,0x00,0x24,0x66,0x66,0x3C,0x18,0x00     ; 86
        .byte 0x00,0x00,0x44,0xD6,0xD6,0xFE,0x6C,0x00     ; 87
        .byte 0x00,0x00,0xC6,0x6C,0x38,0x6C,0xC6,0x00     ; 88
        .byte 0x00,0x00,0x24,0x66,0x66,0x3E,0x06,0x7C     ; 89
        .byte 0x00,0x00,0x7E,0x0C,0x18,0x30,0x7E,0x00     ; 90
        .byte 0x08,0x08,0x08,0x08,0x56,0x55,0x57,0x74     ; 91
        .byte 0x18,0x04,0x08,0x1C,0x56,0x55,0x57,0x74     ; 92
        .byte 0x00,0x00,0x00,0x00,0x7E,0x7E,0xFF,0xFF     ; 93
        .byte 0x18,0x3C,0x18,0x18,0x18,0x18,0x7E,0xFF     ; 94
        .byte 0x22,0x77,0x7F,0x7F,0x3E,0x1C,0x08,0x00     ; 95
